from ruxit.api.base_plugin import RemoteBasePlugin

from prometheus_client.parser import text_string_to_metric_families, Sample
import requests
from requests.models import HTTPBasicAuth

import logging
from urllib.parse import urlparse
from enum import Enum
import socket

log = logging.getLogger(__name__)

METRICS_PATH = "/metrics"
ALERTS_PATH = "/alerts"

relative_metrics = [
    "iris_csp_activity",
    "iris_system_alerts"
]

metrics_to_exclude = [
    "iris_system_alerts_log"
]

class SystemState(Enum):
    GREEN = 0
    YELLOW = 1
    RED = 2
    HUNG = 3

class IrisExtension(RemoteBasePlugin):
    def initialize(self, **kwargs):
        log.setLevel(self.config.get("log_level"))
        iris_url = self.config.get("iris_url")
        while iris_url.endswith("/"):
            iris_url = iris_url[:-1]
        self.iris_url = iris_url
        self.basic_auth = HTTPBasicAuth(self.config.get("username"), self.config.get("password")) if self.config.get("username") else None
        self.execution_count = 0

    def query(self, **kwargs):
        log.info(f"Running IRIS extension query method, execution: {self.execution_count}")

        group_name = self.config.get("group") if self.config.get("group") else "IRIS Instances"
        device_name = self.config.get("alias") if self.config.get("alias") else urlparse(self.iris_url).hostname
        self.group = self.topology_builder.create_group(group_name, group_name)
        self.device = self.group.create_device(device_name, device_name)
        self.device.report_property("Plugin version", kwargs["json_config"]["version"])
        try:
            ip_address = socket.gethostbyname(urlparse(self.iris_url).hostname)
            log.debug(f"Detected IP address: {ip_address}")
            self.device.add_endpoint(ip_address)
        except Exception as e:
            log.warn(f"Unable to determine IP address for {self.iris_url}")

        self.report_metrics()
        self.report_events()

        self.execution_count += 1
    
    def report_metrics(self):
        log.info(f"Reporting metrics...")
        resp = requests.get(f"{self.iris_url}{METRICS_PATH}", auth=self.basic_auth)
        metrics = resp.text
        for family in text_string_to_metric_families(metrics):
            try:
                for sample in family.samples:
                    log.debug(f"Sample: {sample}")
                    if sample.name in metrics_to_exclude:
                        continue
                    elif sample.name == "iris_system_state":
                        self.device.state_metric("iris_system_state", SystemState(sample.value))
                    elif sample.name not in relative_metrics:
                        self.device.absolute(sample.name, sample.value, dimensions=sample.labels)
                    else:
                        self.device.relative(sample.name, sample.value, dimensions=sample.labels)
            except Exception as e:
                log.error(f"Error reporting metric {sample.name}: {e}")

    def report_events(self):
        log.info(f"Reporting events...")
        resp = requests.get(f"{self.iris_url}{ALERTS_PATH}", auth=self.basic_auth)
        alerts = resp.json()
        # alerts = [{"time":"2021-06-22T14:14:35.530Z","severity":"2","message":"System appears to have failed over from node 34e749268eb0"}]
        try:
            for alert in alerts:
                log.debug(f"Alert: {alert}")
                self.device.report_error_event(alert.get("message"), str(alert.get("message")).split('.')[0], {"time": alert.get("time")}) # For title just take up to first period
        except Exception as e:
                log.error(f"Error reporting alert {alert}: {e}")

    def create_mint_line(self, sample: Sample) -> str:
        '''
        Saving this in case ever want/need to use mint instead of extensions framework
        '''
        metric_key = sample.name
        dimensions = ',' + ','.join("{!s}={!r}".format(key,val) for (key,val) in sample.labels.items()) if sample.labels != {} else ""
        value = sample.value
        return f"{metric_key}{dimensions.rstrip(',')} {value}"
